"""
A simple async Python wrapper for nekos.best API.
Copyright (C) 2021  PredaaA

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published
by the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""


class NekosBestBaseError(Exception):
    """Base error of nekosbest client."""


class NotFound(NekosBestBaseError):
    """Raised when API returns a 404."""


class APIError(NekosBestBaseError):
    """Raised when API returns an unexcepted status code."""

    def __init__(self, code: int = None):
        self.code = code


class ClientError(NekosBestBaseError):
    """Raised on Client error."""



"""
A simple async Python wrapper for nekos.best API.
Copyright (C) 2021  PredaaA

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published
by the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from enum import Enum


class Categories(Enum):
    cuddle = "cuddle"
    feed = "feed"
    hug = "hug"
    kiss = "kiss"
    nekos = "nekos"
    pat = "pat"
    poke = "poke"
    slap = "slap"
    tickle = "tickle"

    @classmethod
    def has_value(cls, value: str):
        return value in cls.__members__


class Result:
    def __init__(self, data: dict):
        self.url = data.get("url")



"""
A simple async Python wrapper for nekos.best API.
Copyright (C) 2021  PredaaA

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published
by the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import aiohttp

from .errors import APIError, ClientError, NotFound

BASE_URL = "https://nekos.best"


class HttpClient:
    def __init__(self):
        self.session = aiohttp.ClientSession()

    async def get(self, endpoint: str, **kwargs):
        try:
            async with self.session.get(f"{BASE_URL}/{endpoint}") as resp:
                if resp.status == 404:
                    raise NotFound()
                if resp.status != 200:
                    raise APIError(resp.status)
                return await resp.json(content_type=None)
        except aiohttp.ClientConnectionError:
            raise ClientError()



"""
A simple async Python wrapper for nekos.best API.
Copyright (C) 2021  PredaaA

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published
by the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from .http import HttpClient
from .models import Categories, Result


class Client:
    """Client to make requests to nekos.best API."""

    def __init__(self):
        self.http = HttpClient()

    async def teardown(self):
        """
        |coro|
        
        A function to cleanup our http session.
        """
        await self.http.session.close()

    async def get_image(self, category: str) -> Result:
        """
        |coro|
        
        Returns an image URL of a specific category.

        Parameters
        ----------
        category: str
            The category of image you want to get.
        
        Returns
        -------
        nekosbest.Result
        """
        if not Categories.has_value(category):
            raise TypeError("This isn't a valid category.")

        result = await self.http.get(category)
        return Result(result)
